/**
  *  \file main.cpp
  *  \brief Little Ugly Test Program
  */

#include <iostream>
#include <fstream>
#include <cstdlib>
#include "data.h"
#include "game.h"
#include "util.h"
#include "filenames.h"
#include "program.h"
#include "action.h"
#include "race.h"
#include "selection.h"
#include "loaderstate.h"
#include "net/http.h"
#include "net/smtp.h"
#include "net/url.h"
#include "turnuploadrequest.h"
#include "workerthread.h"
#include "completionhandler.h"
#include "filedownloadrequest.h"
#include "multirequest.h"
#include "basicrequests.h"
#include "initializer.h"
#include "expander.h"
#include "transaction.h"

/** \defgroup console Console test program
    This is the simple console-mode test program. */

/// Ask a yes/no question.
static bool
askYesNo(std::string prompt)
{
    std::string reply;
    std::cout << prompt << " (y/n)? ";
    return getline(std::cin, reply) && reply == "y";
}

static WorkerThread network_thread;

/// Show result of a request.
static void
showRequest(Request* r, std::string prefix)
{
    if (TurnUploadRequest* tu = dynamic_cast<TurnUploadRequest*>(r)) {
        std::cout << prefix << "Turn upload: " << tu->getResultMessage() << "\n";

        const TurnUploadParser& p = tu->getParser();
        std::cout << prefix << "Turn status: " << p.getResult() << "\n"
                  << prefix << "Reg status: " << p.getRegistration() << "\n"
                  << prefix << "Checkturn:\n"
                  << prefix << " | " << strAddPrefix(p.getCheckturnResult(), prefix + " | ") << "\n";
    }
    if (FileDownloadRequest* fd = dynamic_cast<FileDownloadRequest*>(r)) {
        std::cout << prefix << "File download: " << fd->getResultMessage() << "\n";
    }
    if (MultiRequest* m = dynamic_cast<MultiRequest*>(r)) {
        bool processed = true;
        std::cout << prefix << "Multi-Request:\n";
        for (int i = 0; i < m->getNumRequests(); ++i) {
            Request* sub = m->getRequestByNumber(i);
            std::cout << prefix << "  Sub-request '" << sub->getName() << "'\n";
            if (!processed)
                std::cout << prefix << "    Status: Not processed\n";
            else if (sub->getStatus() == Request::StatusSucceeded)
                std::cout << prefix << "    Status: Success\n";
            else
                std::cout << prefix << "    Status: Failure\n";

            showRequest(sub, prefix + "    ");

            if (sub == m->getFailedRequest())
                processed = false;
        }
    }
}

/// Clean up finished requests.
static void
reqGC()
{
    while (Request* r = network_thread.popFinishedRequest()) {
        // Display request
        std::cout << "--- Request '" << r->getName() << "' finished:\n";
        network_thread.getFinishSemaphore().wait();    // must be set
        if (r->getStatus() == Request::StatusSucceeded)
            std::cout << "Status: Success\n";
        else
            std::cout << "Status: Failure\n";

        // Callback
        if (CompletionHandler* h = r->getCompletionHandler())
            h->handleCompletion();

        // request-specific data:
        showRequest(r, "");

        delete r;
    }
}

/// Run an action.
static void
runAction(Action& a, Selection& sel)
{
    sel.selectAction(&a);

    // Build request container
    std::string name = a.getDescription();
    if (sel.getSelectedGame()) {
        name += " for ";
        name += sel.getSelectedGame()->getName();
    }
    MultiRequest mr(name);

    // Figure out commands to perform
    StringList stuff;
    Expander e;
    sel.setVariables(e);
    e.expandToList(stuff, a.getCommandLine());

    if (stuff.empty()) {
        std::cout << "This command is currently not possible: " << name << "\n";
        return;
    }

    // Do it
    for (StringList::iterator i = stuff.begin(); i != stuff.end(); ++i) {
        std::string s = *i;
        std::string verb = eatWord(s);

        // now verb=command, s=arg
        if (verb == "shell") {
            while (s.size() && s[0] == '-')
                eatWord(s);
            std::cout << "Would now run " << s << "\n";
        } else if (verb == "browser") {
            if (s.size()) {
                system(("links '" + s + "'").c_str());
            } else {
                std::cout << ("This action would open a web site, but the URL was not specified.\n");
            }
        } else if (verb == "builtin-download-rst") {
            if (sel.getSelectedGame()) {
                doDownloadResults(*sel.getSelectedGame(), mr);
            }
        } else if (verb == "builtin-download-spec") {
            if (sel.getSelectedGame()) {
                if (s.size()) {
                    doDownloadFile(s, *sel.getSelectedGame(), mr);
                } else {
                    doDownloadSpec(*sel.getSelectedGame(), mr);
                }
            }
        } else if (verb == "builtin-upload-trn") {
            if (sel.getSelectedGame()) {
                doUploadTurns(*sel.getSelectedGame(), mr);
            }
        } else {
            std::cout << "Unknown verb " << verb << "\n";
        }
    }

    if (mr.getNumRequests()) {
        MultiRequest* the_req = new MultiRequest(mr);
        the_req->addReadOperation(sel.getSelectedGame());
        the_req->addReadOperation(&sel.getSelectedAction()->getProgram());
        the_req->acquireLocks();
        network_thread.pushRequest(the_req);
    }
}

static void
doInitialAssignments(Data& d)
{
    Initializer i(d);
    i.assignGameDirectories(false);
    if (i.getGamesWithMissingDirs()) {
        std::cout << "There are " << i.getGamesWithMissingDirs() << " games for which\n"
                  << "I could not assign a game inside the Winplan tree.\n";
        if (askYesNo("Assign outside"))
            i.assignGameDirectories(true);
    }
    if (i.getGamesWithMissingSpecs())
        std::cout << "NOTE: " << i.getGamesWithMissingSpecs() << " games want spec download\n";
}

static void
doMissingDownloads(Data& d)
{
    Initializer i(d);
    MultiRequest mr("Specification Download");
    i.downloadMissingSpecs(mr);
    if (mr.getNumRequests()) {
        std::cout << "Start " << mr.getNumRequests() << " downloads.\n";
        network_thread.pushRequest(new MultiRequest(mr));
    }
}

static void
gameMenu(Data& d, Game& g)
{
    std::vector<Program*> p;
    VectorValueEnumerator<Program> pi(p);
    d.enumeratePrograms(pi);

    std::vector<Action*> a;
    VectorValueEnumerator<Action> ai(a);
    d.enumerateSimpleActions(g, ai);

    std::vector<Race*> r;
    VectorValueEnumerator<Race> ri(r);
    g.enumerateRaces(ri);

    Selection sel;
    sel.selectGame(&g);

    std::cout << "-- Game " << g.getName() << " --\n";
    if (sel.getSelectedRace())
        std::cout << "r: race (current is " << sel.getSelectedRace()->getRaceNumber() << ")\n";
    else
        std::cout << "r: race (none selected)\n";
    for (unsigned int i = 0; i < a.size(); ++i)
        std::cout << i << ": " << a[i]->getDescription() << " ("
                  << a[i]->getName() << ", using "
                  << a[i]->getProgram().getName() << ")\n";
    for (unsigned int i = 0; i < p.size(); ++i)
        std::cout << "p" << i << ": " << p[i]->getName() << " ->\n";
    while (1) {
        std::string reply;
        std::cout << "Main Game> ";
        if (!getline(std::cin, reply))
            break;
        int n;
        reqGC();
        if (reply == "q") {
            break;
        } else if (reply == "r") {
            for (unsigned int i = 0; i < r.size(); ++i)
                std::cout << i << ": race " << r[i]->getRaceNumber() << (r[i] == sel.getSelectedRace() ? " <-\n" : "\n");
            if (r.size() == 0)
                std::cout << "No races defined for this game\n";
            std::cout << "Main Game Race> ";
            if (getline(std::cin, reply) && atoi(reply, n) && n >= 0 && n < int(r.size()))
                sel.selectRace(r[n]);
        } else if (atoi(reply, n) && n >= 0 && n < int(a.size())) {
            runAction(*a[n], sel);
        } else if (reply.size() && reply[0] == 'p' && atoi(reply.substr(1), n) && n >= 0 && n < int(p.size())) {
            std::vector<Action*> pa;
            VectorValueEnumerator<Action> pai(pa);
            p[n]->enumerateActions(pai);
            std::cout << "::: Program " << p[n]->getName() << " :::\n";
            for (unsigned int i = 0; i < pa.size(); ++i)
                std::cout << i << ": " << pa[i]->getDescription() << "\n";
            std::cout << "Main Game Program> ";
            if (getline(std::cin, reply) && atoi(reply, n) && n >= 0 && n < int(pa.size())) {
                reqGC();
                runAction(*pa[n], sel);
            }
        } else if (reply != "") {
            std::cout << "huh?\n";
        }
    }
}

static void
loadFile(Data& d)
{
    std::string s, reply;
    std::cout << "Filename?";
    if (getline(std::cin, s)) {
        std::ifstream in(s.c_str());
        if (!in) {
            std::cerr << "can't open\n";
        } else {
            try {
                Data tmp;
                LoaderState st;
                Transaction ta;
                WriteOperationEnumerator<Program> progwrite(ta);
                WriteOperationEnumerator<Game> gamewrite(ta);
                d.enumeratePrograms(progwrite);
                d.enumerateGames(gamewrite);
                if (!ta.tryAcquireLocks()) {
                    std::cout << "Unable to load new file while background transaction running.\n";
                    return;
                }

                st.setBaseDirectory(getDirectoryName(s));
                tmp.read(in, st);
                std::cout << "File contains " << st.getNumItems(st.Globals) << " global config definitions,\n"
                          << "  " << st.getNumItems(st.Programs) << " program definitions,\n"
                          << "  " << st.getNumItems(st.Games) << " game definitions.\n";

                st.setAccept(st.Globals,  st.getNumItems(st.Globals)  != 0 && askYesNo("Read globals"));
                st.setAccept(st.Programs, st.getNumItems(st.Programs) != 0 && askYesNo("Read programs"));
                st.setAccept(st.Games,    st.getNumItems(st.Games)    != 0 && askYesNo("Read games"));
                if (st.isAcceptAny()) {
                    in.clear(); // weil wir am Ende waren
                    in.seekg(0);
                    d.read(in, st);
                    doInitialAssignments(d);
                }
            }
            catch (std::exception& e) {
                std::cout << "Load aborted: " << e.what() << "\n";
            }
        }
    }
}

static void
newGame(Data& d)
{
    std::string reply;
    std::string name;
    std::cout << "Game name?";
    if (!getline(std::cin, name))
        return;

    if (d.getGameByName(name)) {
        std::cout << "haben wir schon.\n";
        return;
    }

    std::string dir = d.getNewGameDirectory(name, false);
    if (!dir.size()) {
        std::cout << "You have Winplan, and I cannot find a free game slot.\n";
        if (!askYesNo("Continue and make the game outside the Winplan tree?"))
            return;
        dir = d.getNewGameDirectory(name, true);
    }
    std::cout << "Game will be in '" << dir << "'\n";

    Game* g = new Game(name);
    d.addGame(g);
    g->setLocalDirectory(dir);
    g->setStatus(Game::StatusNeedSpecs);

    // Noch eine Rasse anlegen
    int nr;
    Race* r = 0;
    std::cout << "What race do you play?";
    if (getline(std::cin, reply) && atoi(reply, nr) && nr > 0 && nr <= 11) {
        r = new Race(nr);
        g->addRace(r);
    }

    // Lokal?
    std::cout << "If this is a local game, enter host directory:";
    if (getline(std::cin, dir) && dir.size()) {
        LoaderState st;
        g->assign("hostdir", dir, st);
        if (r) {
            r->assign("uploadurl", getFileUrl(dir), st);
            r->assign("downloadurl", getFileUrl(makeFileName(dir, "player" + itoa(nr) + ".rst")), st);
            r->assign("downloadurl+", getFileUrl(makeFileName(dir, "util" + itoa(nr) + ".dat")), st);
        }
    }
}

static void
netConfig()
{
    Url u;
    std::string reply;
    std::string reply2;
    int port;

    // proxy
    do {
        std::cout << "Proxy (just ENTER for none)?";
        if (!getline(std::cin, reply))
            return;
    } while (reply.size() && !u.parse(reply));
    Http::setProxy(u);

    // mail address
    std::cout << "Mail address?";
    getline(std::cin, reply);
    Smtp::setFromAddress(reply);

    // SMTP host
    std::cout << "SMTP server?";
    getline(std::cin, reply);
    std::cout << "SMTP port (ENTER for default)?";
    getline(std::cin, reply2);
    port = -1;
    atoi(reply2, port);
    Smtp::setServer(reply, port);
}

static void
mainMenu(Data& d)
{
    std::string reply;
    while (1) {
        std::cout << "=== Game List ===\n";
        std::vector<Game*> g;
        {
            VectorValueEnumerator<Game> vi(g);
            d.enumerateGames(vi);
        }
        std::cout << "load: load definition\n"
                  << "new: new game\n"
                  << "assign: assign missing local dirs\n"
                  << "dl: download missing specs\n"
                  << "q: quit\n"
                  << "print: print config file to stdout\n"
                  << "nc: network config\n"
                  << "rm: remove game\n";
        for (std::vector<Game*>::size_type i = 0; i < g.size(); ++i)
            std::cout << i << ": " << g[i]->getName() << "    in " << g[i]->getLocalDirectory() << "\n";
        std::cout << "Main> ";
        if (!getline(std::cin, reply))
            return;
        reqGC();
        if (reply == "print") {
            d.write(std::cout);
        } else if (reply == "load") {
            loadFile(d);
        } else if (reply == "q") {
            break;
        } else if (reply == "assign") {
            doInitialAssignments(d);
        } else if (reply == "dl") {
            doMissingDownloads(d);
        } else if (reply == "rm") {
            int nr;
            std::cout << "Which one?";
            if (getline(std::cin, reply) && atoi(reply, nr) && nr >= 0 && nr < int(g.size()))
                d.deleteGame(g[nr]);
        } else if (reply == "nc") {
            netConfig();
        } else if (reply == "new") {
            newGame(d);
        } else {
            int num;
            if (atoi(reply, num) && num >= 0 && num < int(g.size()))
                gameMenu(d, *g[num]);
            else
                std::cout << "h?\n";
        }
    }
}

int main(int argc, char** argv)
{
    Data d;
    std::string filename = getSettingsFileName();
    if (argc == 2) {
        filename = argv[1];
    }

    {
        std::ifstream in(filename.c_str());
        if (in) {
            LoaderState st;
            st.setBaseDirectory(getDirectoryName(filename));
            d.read(in, st);
        }
    }

    d.ensureDefaultBindings(false);

    network_thread.start();
    mainMenu(d);

    {
        std::string bak = filename + ".bak";
        std::remove(bak.c_str());
        std::rename(filename.c_str(), bak.c_str());
        std::ofstream out(filename.c_str());
        if (out) {
            d.write(out);
        }
    }
}
