/**
  *  \file filenames.cpp
  *  \brief File and File Name Handling
  */

#include <cstdlib>
#include <fstream>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include "filenames.h"

/** Extract path of a file name. */
std::string
getDirectoryName(std::string s)
{
    std::string::size_type n = s.rfind('/');
    if (n == s.npos)
        return std::string("");
    return s.substr(0, n);
}

/** Extract non-directory part of a file name. */
std::string
getBaseName(std::string s)
{
    std::string::size_type n = s.rfind('/');
    if (n == s.npos)
        return s;
    return s.substr(n);
}

/** Given a parsed file:// url, get the actual file name.
    \param host u.getHost()
    \param s    u.getPath()
    \return file name */
std::string
getFileNameFromUrl(std::string host, std::string s)
{
    (void) host;
    if (s.empty())
        s = "/";
    return s;
}

/** Convert file name into file:// url. The identity should be that
    getFileUrl(getFileNameFromUrl(...)) yields a file name identifying
    the same file. */
std::string
getFileUrl(std::string name)
{
    if (name.size() && name[0] == '/')
        return "file://" + name;
    else
        return "file:///" + name;
}

/** Make file name. Returns the file name that will be accessed when
    we open \c name while sitting in working directory \c basedir. */
std::string
makeFileName(std::string basedir, std::string name)
{
    if (name.size() == 0)
        return basedir;

    if (name[0] == '/')
        return name;

    if (basedir.size() != 0 && basedir[basedir.size()-1] != '/')
        basedir += "/";

    return basedir + name;
}

std::string
getSettingsFileName()
{
    const char* p = std::getenv("HOME");
    if (p) {
        return makeFileName(p, ".vgapintegrator");
    } else {
        return ".vgapintegrator";
    }
}

/** Check for a file's existance. Checks whether \c name names an
    existing object in the file system (can also be a directory, link,
    etc.)
    \return true if file exists */
bool
isExistingFileName(std::string name)
{
    struct stat x;
    return stat(name.c_str(), &x) == 0;
}

/** Create directory tree.
    \param name Name of directory to create, can contain any number of components
    \return true iff successful */
bool
createDirectories(std::string name)
{
    // remove trailing slashes
    while (name.size() && name[name.size()-1] == '/')
        name.erase(name.size()-1);

    if (name.size() == 0) {
        // attempt to create root
        return true;
    } else {
        if (mkdir(name.c_str(), 0777) != 0) {
            if (errno == ENOENT) {
                std::string::size_type n = name.rfind('/');
                if (n == std::string::npos || n == 0) {
                    // no other directory component
                    return false;
                } else {
                    // create subdirectories and retry
                    createDirectories(std::string(name, 0, n));
                    return mkdir(name.c_str(), 0777) == 0;
                }
            } else if (errno == EEXIST) {
                // directory already exists
                return true;
            } else {
                // other error
                return false;
            }
        } else {
            // mkdir succeeded
            return true;
        }
    }
}

#ifdef MKDIRTEST
#include <iostream>
int main(int, char** argv)
{
    while (*++argv) {
        std::cout << "mkdir(" << *argv << ") = "
                  << createDirectories(*argv) << "\n";
    }
    return 0;
}
#endif
