/**
  *  \file sem.h
  *
  *  This file is called "sem.h" to avoid ambiguities with
  *  <semaphore.h>.
  */
#ifndef SEM_H_INCLUDED
#define SEM_H_INCLUDED

#include <semaphore.h>

//@{
///\ingroup unix

/** Counting semaphore. We do not need nestable mutexes, so simple
    semaphores are enough.

    The Unix implementation uses POSIX (not System V) semaphores. */
class Semaphore {
    sem_t m_TheSemaphore;

    Semaphore(const Semaphore&);
    Semaphore& operator=(const Semaphore&);
 public:
    /** Initialize semaphore.
        \param value initial value, i.e. number of times wait() can
        be called without actually waiting. */
    explicit Semaphore(int value)
        { sem_init(&m_TheSemaphore, 0, value); }

    ~Semaphore()
        { sem_destroy(&m_TheSemaphore); }

    /** Release semaphore. Increments the semaphore. */
    void release()
        { sem_post(&m_TheSemaphore); }

    /** Obtain semaphore. If the semaphore is zero, waits until
        someone calls release(), otherwise just decrements the
        semaphore. */
    void wait()
        { sem_wait(&m_TheSemaphore); }

    /** Try to obtain the semaphore. If the semaphore is zero, returns
        false. Otherwise, decrements it and returns true. */
    bool tryWait()
        { return sem_trywait(&m_TheSemaphore) == 0; }
};

//@}

#endif
