/**
  *  \file basicrequests.cpp
  *  \brief Basic Network Requests
  */

#include "basicrequests.h"
#include "game.h"
#include "filedownloadrequest.h"
#include "race.h"
#include "turnuploadrequest.h"

/** Download file
    \param url  URL to download
    \param g    Game to download files for
    \param exec Collects download requests */
void
doDownloadFile(std::string url, Game& g, RequestConsumer& exec)
{
    exec.pushRequest(new FileDownloadRequest(url, g));
}

/** Download files.
    \param urls  List of URLs
    \param g     Game to download files for
    \param exec  Collects the individual requests
    \return true if any URLs were added to the request */
bool
doDownloadFiles(const StringList& urls, Game& g, RequestConsumer& exec)
{
    for (StringList::const_iterator i = urls.begin(); i != urls.end(); ++i)
        exec.pushRequest(new FileDownloadRequest(*i, g));
    return !urls.empty();
}


/** Download specification files for a game. This isn't particularily
    elegant, because it does the iteration business itself instead of
    having Expander do it, but I couldn't think of a more clever way
    :-( */
void
doDownloadSpec(Game& g, RequestConsumer& exec)
{
    class SpecDownloader : public ValueEnumerator<Race> {
        Game& g;
        RequestConsumer& exec;
        bool did;
     public:
        SpecDownloader(Game& g, RequestConsumer& exec)
            : g(g), exec(exec), did(false)
            { }
        void handleItem(Race& r)
            { did |= doDownloadFiles(r.getSpecificationUrls(), g, exec); }
        bool didDownload() const
            { return did; }
    };
    
    // download game spec, and all race's specs
    bool did = doDownloadFiles(g.getSpecificationUrls(), g, exec);

    SpecDownloader sdl(g, exec);
    g.enumerateRaces(sdl);

    if (did || sdl.didDownload()) {
        // FIXME: mark game as spec-download-done if request finished
    }
}

void
doUploadTurns(Game& g, RequestConsumer& exec)
{
    class TurnUploader : public ValueEnumerator<Race> {
        Game& g;
        RequestConsumer& exec;
     public:
        TurnUploader(Game& g, RequestConsumer& exec)
            : g(g), exec(exec)
            { }

        void handleItem(Race& r)
            { exec.pushRequest(new TurnUploadRequest(g, r)); }
    };

    TurnUploader tu(g, exec);
    g.enumerateRaces(tu);
}

void
doDownloadResults(Game& g, RequestConsumer& exec)
{
    class ResultDownloader : public ValueEnumerator<Race> {
        Game& g;
        RequestConsumer& exec;
     public:
        ResultDownloader(Game& g, RequestConsumer& exec)
            : g(g), exec(exec)
            { }
        void handleItem(Race& r)
            { doDownloadFiles(r.getDownloadUrls(), g, exec); }
    };

    ResultDownloader rd(g, exec);
    g.enumerateRaces(rd);
}
