/**
  *  \file evaluator.cpp
  *  \brief Evaluating finished requests
  */

#include "evaluator.h"
#include "turnuploadrequest.h"
#include "filedownloadrequest.h"
#include "multirequest.h"
#include "nls.h"

Evaluator::Evaluator()
    : m_Description(),
      m_Status(StatusSucceeded)
{ }

void
Evaluator::evaluateRequest(Request* r)
{
    // set default
    if (r->getStatus() == Request::StatusFailed)
        setState("", StatusFailed);
    else
        setState("", StatusSucceeded);

    if (TurnUploadRequest* t = dynamic_cast<TurnUploadRequest*>(r)) {
        if (t->getStatus() == Request::StatusFailed) {
            // Network error etc.
            setState(t->getResultMessage(), StatusFailed);
        } else {
            // Network ok, check result
            const TurnUploadParser& p = t->getParser();
            TurnUploadParser::TurnResult turnres;
            TurnUploadParser::TurnRegistration turnreg;
            std::string s = p.explainRegistration(p.getRegistration(), turnreg);
            if (turnreg == TurnUploadParser::RegBad) {
                setState(s, StatusTurnError);
            } else {
                s = p.explainResult(p.getResult(), turnres);
                if (turnres == TurnUploadParser::ResultRed || turnres == TurnUploadParser::ResultBad)
                    setState(s, StatusTurnError);
                else if (turnres == TurnUploadParser::ResultYellow)
                    setState(s, StatusTurnWarning);
                else if (turnres == TurnUploadParser::ResultUnknown)
                    setState(t->getResultMessage(), StatusSucceeded);
                else
                    setState(s, StatusSucceeded);
            }
        }
    } else if (FileDownloadRequest* f = dynamic_cast<FileDownloadRequest*>(r)) {
        // FIXME
        (void) f;
    } else if (MultiRequest* m = dynamic_cast<MultiRequest*>(r)) {
        for (int i = 0; i < m->getNumRequests(); ++i) {
            evaluateRequest(m->getRequestByNumber(i));
            if (m->getRequestByNumber(i) == m->getFailedRequest())
                break;
        }
    } else {
        // generic
    }
}

std::string
Evaluator::getDescription() const
{
    if (m_Description.size())
        return m_Description;

    static const char* names[] = { N_("Success"), N_("Warning"), N_("Error"), N_("Failure") };
    return _(names[m_Status]);
}

void
Evaluator::setState(std::string desc, Status status)
{
    if (status > m_Status || (status == m_Status && m_Description.size() == 0)) {
        m_Status      = status;
        m_Description = desc;
    }
}
