/**
  *  \file expander.cpp
  *  \brief Expander class implementation
  *  \author Stefan Reuther
  */

#include <iostream>
#include "expander.h"

/** Constructor. Initially, no variables are defined. */
Expander::Expander()
{ }

/** Destructor. */
Expander::~Expander()
{ }

/** Add variable.
    \param key Name
    \param value Value */
void
Expander::handleItem(std::string key, std::string, std::string value)
{
    /* We ignore empty items. In particular, we automatically refuse
       to run utilities for games that don't have a %localdir%. */
    if (value.size())
        addVariable(key, value);
}

/** Add all variables from a PropertyObject. */
void
Expander::addObject(PropertyObject& obj)
{
    obj.enumerate(*this);
}

/** Add a single variable. */
void
Expander::addVariable(std::string name, std::string value)
{
    if (name.size() && name[name.size()-1] == '+') {
        m_Variables[std::string(name, 0, name.size()-1)].push_back(value);
    } else {
        m_Variables[name].assign(1, value);
    }
}

/** Expand string. Variable references are specified using \%name%. A
    single percent sign can be specified using \%\%. Non-existant
    variables cause this function to return an empty string. In case
    of list variables, this always fills in the first value of the
    variable. Thus, this is a simpler version of expandToList().

    \param s string to expand
    \return expanded string. Returns empty string if a non-existant
    variable was referenced */
std::string
Expander::expand(std::string s) const
{
    std::string result;
    while (s.size()) {
        std::string::size_type x = s.find('%');
        if (x == s.npos) {
            result += s;
            break;
        }
        result.append(s, 0, x);
        s.erase(0, x+1);

        x = s.find('%');
        std::string varname;
        if (x == s.npos) {
            varname = s;
            s.erase();
        } else {
            varname.assign(s, 0, x);
            s.erase(0, x+1);
        }
        if (varname.size() == 0) {
            result += "%";
        } else {
            std::map<std::string, StringList>::const_iterator i = m_Variables.find(varname);
            if (i != m_Variables.end() && !i->second.empty())
                result += i->second.front();
            else
                return std::string();
        }
    }
    return result;
}

/** Expand string into list. Variable references are specified using
    \%name%. A single percent sign can be specified using \%\%. In
    case of list variables, fills in all possible values (thus
    computing cross products if multiple list variables are referenced
    in one expansion, and returning an empty result set if a variable
    is not defined).

    \param s      [in] string to expand
    \param result [out] Receives the result */
void
Expander::expandToList(StringList& result, std::string s) const
{
    result.assign(1, std::string());
    while (s.size()) {
        std::string::size_type x = s.find('%');

        // append this part to each string
        for (StringList::iterator i = result.begin(); i != result.end(); ++i)
            i->append(s, 0, x);
        if (x == s.npos)
            break;

        // erase part and '%'
        s.erase(0, x+1);

        // find next '%' and figure out variable name
        x = s.find('%');
        std::string varname;
        if (x == s.npos) {
            varname = s;
            s.erase();
        } else {
            varname.assign(s, 0, x);
            s.erase(0, x+1);
        }
        if (varname.size() == 0) {
            // "%%"
            for (StringList::iterator i = result.begin(); i != result.end(); ++i)
                i->append("%");
        } else {
            std::map<std::string, StringList>::const_iterator i = m_Variables.find(varname);
            if (i != m_Variables.end() && !i->second.empty()) {
                // it's a list. Compute cross product.
                StringList tmp;
                for (StringList::const_iterator outer = result.begin(); outer != result.end(); ++outer)
                    for (StringList::const_iterator inner = i->second.begin(); inner != i->second.end(); ++inner)
                        tmp.push_back(*outer + *inner);
                tmp.swap(result);
            } else {
                // empty or non-existant
                result.clear();
                break;
            }
        }
    }
}

#ifdef EXPANDERTEST
#include <iostream>
int main()
{
    std::string line;
    Expander x;
    while (getline(std::cin, line)) {
        std::string::size_type p = line.find('=');
        if (p != line.npos) {
            x.addVariable(line.substr(0, p), line.substr(p+1));
        } else {
            std::string s = x.expand(line);
            if (s.size())
                std::cout << "String-expands into '" << s << "'\n";
            else
                std::cout << "String-expands into nothing\n";

            StringList l;
            x.expandToList(l, line);
            if (l.size()) {
                std::cout << "List-expands into:\n";
                for (StringList::iterator i = l.begin(); i != l.end(); ++i)
                    std::cout << "  '" << *i << "'\n";
            } else {
                std::cout << "List-expands into nothing\n";
            }
        }
    }
}
#endif
