/**
  *  \file filedownloadrequest.cpp
  *  \brief File Download Request class
  */

#include <fstream>
#include "filedownloadrequest.h"
#include "nls.h"
#include "net/url.h"
#include "net/http.h"
#include "net/mimebuilder.h"
#include "game.h"
#include "filenames.h"
#include "util.h"

/** Constructor.
    \param url  [in] URL to download
    \param game [in] Game to download file for */
FileDownloadRequest::FileDownloadRequest(std::string url, Game& game)
    : m_Url(url), m_Game(game)
{ }

FileDownloadRequest::~FileDownloadRequest()
{ }

void
FileDownloadRequest::run()
{
    Url u;
    if (!u.parse(m_Url)) {
        setResultMessage(_("Invalid URL"));
        setStatus(StatusFailed);
        return;
    }

    if (u.getProtocol() == "file") {
        doCopy(u);
    } else if (u.getProtocol() == "http" || u.getProtocol() == "http-post") {
        doDownload(u);
    } else {
        setResultMessage(_("Unsupported download protocol"));
        setStatus(StatusFailed);
    }
}

/** Download file by HTTP. */
void
FileDownloadRequest::doDownload(Url& u)
{
    std::string tmpname = makeFileName(m_Game.getLocalDirectory(), "__vgapi.tmp");
    std::ofstream out(tmpname.c_str(), std::ofstream::binary);

    Http downloader;
    bool result;
    if (u.getProtocol() == "http") {
        // GET
        result = downloader.downloadFile(u, out);
    } else {
        // POST
        MimeBuilder mb("multipart/form-data");
        MimeBuilder builder("multipart/form-data");
        MimeParameterEncoder mpe(mb);
        u.trimParameters(mpe);
        mb.addBoundary();
        mb.finish();
        result = downloader.post(u, mb, out);
    }
    out.close();

    if (!result) {
        // Network error
        setResultMessage(_("Network error: ") + downloader.getErrorMessage()
                         + ", " + downloader.getServerErrorMessage());
        setStatus(StatusFailed);
        std::remove(tmpname.c_str());
    } else if (!downloader.isSuccessfulReply()) {
        // HTTP error
        setResultMessage(_("Server sent error: ") + downloader.getReplyLine());
        setStatus(StatusFailed);
        std::remove(tmpname.c_str());
    } else {
        // Figure out the name of the file
        std::string outname;
        outname = getParameter(downloader.getHeader("Content-Disposition"), "filename");
        if (outname.size() == 0)
            outname = getParameter(downloader.getHeader("Content-Type"), "filename");
        if (outname.size() == 0)
            outname = u.getBaseName();

        outname = getBaseName(outname);

        setResultMessage(_("File ") + outname + _(" received successfully"));

        // uh-oh.
        // FIXME: error handling
        std::string n = makeFileName(m_Game.getLocalDirectory(), outname);
        std::remove(n.c_str());
        std::rename(tmpname.c_str(), n.c_str());
    }
}

/** Copy file (file:// URL). */
void
FileDownloadRequest::doCopy(Url& u)
{
    std::string srcname = getFileNameFromUrl(u.getHost(), u.getPath());
    std::string destname = makeFileName(m_Game.getLocalDirectory(), u.getBaseName());

    std::ifstream in(srcname.c_str(), std::ifstream::binary);
    if (!in) {
        setResultMessage(_("Can't open input file '") + srcname + "'");
        setStatus(StatusFailed);
        return;
    }

    std::ofstream out(destname.c_str(), std::ofstream::binary);
    if (!out) {
        setResultMessage(_("Can't create '") + destname + "'");
        setStatus(StatusFailed);
        return;
    }

    out << in.rdbuf();
    setResultMessage(_("File copied successfully"));
}

std::string
FileDownloadRequest::getName()
{
    return _("Download ") + m_Url + _(" for ") + m_Game.getName();
}
