/**
  *  \file game.cpp
  *  \brief Game class implementation
  *  \author Stefan Reuther
  */

#include <stdexcept>
#include <cstring>
#include <fstream>
#include "game.h"
#include "race.h"
#include "util.h"
#include "nls.h"
#include "expander.h"
#include "filenames.h"

const char*const Game::status_table[] = {
    "new",
    "needspecs",
    "ok",
    0
};

/** Constructor.
    \param name Game name */
Game::Game(std::string name)
    : m_Name(name), m_Status(StatusNew)
{
    std::memset(m_RaceNames, 0, sizeof(m_RaceNames));
}

/** Destructor. */
Game::~Game()
{ }

void
Game::enumerate(KVEnumerator& e)
{
    e.handleItem("-",        _("Name of the game"), m_Name);
    e.handleItem("localdir", _("Local directory where this game's files are stored"), m_LocalDir);
    e.handleItem("gameurl",  _("URL of a web site for the game"), m_GameUrl);
    e.handleListItem("specurl",  _("URL of specification files"), m_SpecUrls);
    e.handleItem("hostdir",  _("Host directory for locally-hosted games"), m_HostDir);
    e.handleItem("used",     _("List of used features/add-ons in this game"), m_UsedFeatures.toString());
    e.handleItem("status",   _("Status of this game (internal)"), status_table[m_Status]);
}

bool
Game::assign(std::string name, std::string value, LoaderState& /*st*/)
{
    if (name == "-") {
        m_Name = value;
        return true;
    }
    if (name == "localdir") {
        m_LocalDir = value;
        return true;
    }
    if (name == "gameurl") {
        m_GameUrl = value;
        return true;
    }
    if (name == "specurl") {
        m_SpecUrls.clear();
        m_SpecUrls.push_back(value);
        return true;
    }
    if (name == "specurl+") {
        m_SpecUrls.push_back(value);
        return true;
    }
    if (name == "hostdir") {
        m_HostDir = value;
        return true;
    }
    if (name == "used") {
        m_UsedFeatures.assignFromString(value);
        return true;
    }
    if (name == "status") {
        return stringToEnum(value, status_table, m_Status);
    }
    return false;
}

PropertyObject*
Game::createSubtree(std::string name, std::string value, LoaderState& /*st*/)
{
    if (name == "race") {
        int nr;
        if (atoi(value, nr) && nr > 0 && nr <= 11) {
            Race* r = getRaceByNumber(nr);
            if (!r) {
                r = new Race(nr);
                addRace(r);
            }
            return r;
        } else {
            fail(_("Invalid race number"));
        }
    }
    return 0;
}

/** Add a new race. */
void
Game::addRace(Race* r)
{
    safeAppend(m_Races, r);
}

/** Find race by number.
    \return race, or 0 if no such race in this game */
Race*
Game::getRaceByNumber(int nr)
{
    for (std::vector<Race*>::iterator i = m_Races.begin(); i != m_Races.end(); ++i)
        if ((*i)->getRaceNumber() == nr)
            return *i;
    return 0;
}

/** Get default race. This is the race which is selected when this
    game is selected. \todo make this configurable */
Race*
Game::getDefaultRace()
{
    if (m_Races.size())
        return m_Races.front();
    else
        return 0;
}

/** Enumerate all races. */
void
Game::enumerateRaces(ValueEnumerator<Race>& r)
{
    for (std::vector<Race*>::iterator i = m_Races.begin(); i != m_Races.end(); ++i)
        r.handleItem(**i);
}

/** Load race names file. Call this after changes to the local
    directory. */
void
Game::loadRaceNames()
{
    std::memset(m_RaceNames, 0, sizeof(m_RaceNames));
    if (m_LocalDir.size()) {
        std::ifstream file(makeFileName(m_LocalDir, "race.nm").c_str(),
                           std::ifstream::binary | std::ifstream::in);
        if (file)
            file.read(m_RaceNames, sizeof(m_RaceNames));
    }
}

/** Get race name.
    \param race Race number (1..11)
    \param type Type of race name to obtain
    \return name if known, empty string otherwise */
std::string
Game::getRaceName(int race, RaceName type) const
{
    static const int size[] = { 30, 20, 12 };
    static const int origin[] = { 0, 11*30, 11*(30+20) };

    if (race > 0 && race <= 11) {
        std::string result(&m_RaceNames[origin[type] + size[type] * (race-1)], size[type]);
        std::string::size_type i = result.find('\0');
        if (i != result.npos)
            i = result.size();
        while (i != 0 && result[i-1] == ' ')
            --i;
        result.erase(i);
        return result;
    } else {
        return std::string();
    }
}
