/**
  *  \file loaderstate.cpp
  *  \brief LoaderState class implementation
  *  \author Stefan Reuther
  */

#include "loaderstate.h"
#include "filenames.h"

/** Constructor. By default, we accept everything. */
LoaderState::LoaderState()
{
    setAcceptAll();
    start();
}

/** Destructor. */
LoaderState::~LoaderState()
{ }

/** Start reading. This resets the counters. */
void
LoaderState::start()
{
    for (int i = 0; i < MAX; ++i)
        m_Count[i] = 0;
}

/** Set acceptance.
    \param what  [in] Thing to configure
    \param allow [in] Whether these things are allowed to be read */
void
LoaderState::setAccept(What what, bool allow)
{
    m_Accept[what] = allow;
}

/** Set acceptance. Allows acceptance of all things. */
void
LoaderState::setAcceptAll()
{
    for (int i = 0; i < MAX; ++i)
        m_Accept[i] = true;
}

/** Check whether item is allowed. Also increases the counter. Call
    this when you have an item of type \c w.
    \param w Item to check
    \return true if it's allowed to assimilate that in our main data */
bool
LoaderState::isAccept(What w)
{
    ++m_Count[w];
    return m_Accept[w];
}

/** Set base directory. This directory is used to expand relative
    directories. Program definitions usually have "directory ." in
    them, which is expanded to the directory containing the program
    definition file when assimilating it.
    \param dir Base directory. Should be absolute. */
void
LoaderState::setBaseDirectory(std::string dir)
{
    m_Basedir = dir;
}

/** Expand directory.
    \param dir Relative directory.
    \return absolute version of \c dir. */
std::string
LoaderState::expandDirectory(std::string dir)
{
    return makeFileName(m_Basedir, dir);
}

/** Get number of items.
    \param w Items to query
    \return how many items of type \c w were contained in the
    previously-loaded file */
int
LoaderState::getNumItems(What w)
{
    return m_Count[w];
}

/** Check whether we're configured to accept anything. Only in this
    case we need to read the file. */
bool
LoaderState::isAcceptAny()
{
    for (int i = 0; i < MAX; ++i)
        if (m_Accept[i])
            return true;
    return false;
}
