/**
  *  \file multirequest.cpp
  *  \brief MultiRequest container class
  */

#include "multirequest.h"
#include "util.h"

/** Constructor.
    \param name Name of this request */
MultiRequest::MultiRequest(std::string name)
    : m_Name(name),
      m_FailedRequest(0)
{ }

/** Pseudo copy constructor. This copies other's request queue and
    name, emptying the former in the process. The intent is to use
    it as
    <code>
      MultiRequest mr("name");
      mr.pushRequest(new X());
      mr.pushRequest(new X());
      network_thread.pushRequest(new MultiRequest(mr));
    </code>
    This guarantees leak-freeness.

    \param other Request to initialize with. */
MultiRequest::MultiRequest(MultiRequest& other)
    : Request(),
      RequestConsumer(),
      Transaction(),
      m_Name(other.m_Name),
      m_FailedRequest(0)
{
    m_Requests.swap(other.m_Requests);
}

MultiRequest::~MultiRequest()
{
    safeClear(m_Requests);
}

/** Main entry point. */
void
MultiRequest::run()
{
    for (std::vector<Request*>::iterator i = m_Requests.begin(); i != m_Requests.end(); ++i) {
        Request* r = *i;
        r->setStatus(StatusRunning);
        try {
            r->run();
            if (r->getStatus() == StatusRunning)
                r->setStatus(StatusSucceeded);
        }
        catch (...) {
            r->setStatus(StatusFailed);
        }
        if (r->getStatus() == StatusFailed) {
            m_FailedRequest = r;
            setStatus(StatusFailed);
            break;
        }
    }
}

std::string
MultiRequest::getName()
{
    return m_Name;
}

/** Add new request. Note that this routine must not be called once
    run() started running (i.e. once this has been handed to a
    WorkerThread). */
void
MultiRequest::pushRequest(Request* req)
{
    req->setStatus(StatusWaiting);
    safeAppend(m_Requests, req);
}
