/**
  *  \file transaction.h
  */
#ifndef TRANSACTION_H_INCLUDED
#define TRANSACTION_H_INCLUDED

#include <vector>
#include "sem.h"
#include "enumerator.h"

class Transaction;

/** Base class for transactable objects. Contains reader-writer-lock
    reference counts. All objects that want to be used with
    Transaction must derive from this. */
class Transactable {
    int m_NumReaders;
    int m_NumWriters;
 public:
    Transactable()
        : m_NumReaders(0), m_NumWriters(0)
        { }
    virtual ~Transactable()
        { }

    friend class Transaction;
};

/** Transaction arbiter. This provides interlocking between multiple
    threads that try to access an object set, using reader-writer-lock
    semantics.

    Since we expect few threads only, we use
    - one semaphore per thread, not object
    - no provisions for particularily fair scheduling */
class Transaction {
    enum Operation {
        ReadOperation,
        WriteOperation
    };
    struct Element {
        Transactable* m_Object;
        Operation m_Operation;

        Element(Transactable* obj, Operation op)
            : m_Object(obj), m_Operation(op)
            { }
    };
    std::vector<Element> m_Operations;

    bool m_Active;

    Semaphore m_WaitLock;

    Transaction* m_Next;
    Transaction** m_Prev;

    static Transaction* s_WaitingTransactions;
    static Semaphore s_GlobalLock;

    bool checkCanStart();

    void doStart();
    void doStop();
 public:
    Transaction();

    virtual ~Transaction();

    void addReadOperation(Transactable* obj);

    void addWriteOperation(Transactable* obj);

    void acquireLocks();

    bool tryAcquireLocks();

    void releaseLocks();
};

template<typename T>
class ReadOperationEnumerator : public ValueEnumerator<T> {
    Transaction& m_Transaction;
 public:
    ReadOperationEnumerator(Transaction& t)
        : m_Transaction(t) { }
    void handleItem(T& ele)
        { m_Transaction.addReadOperation(&ele); }
};

template<typename T>
class WriteOperationEnumerator : public ValueEnumerator<T> {
    Transaction& m_Transaction;
 public:
    WriteOperationEnumerator(Transaction& t)
        : m_Transaction(t) { }
    void handleItem(T& ele)
        { m_Transaction.addWriteOperation(&ele); }
};

#endif
