/**
  *  \file turnuploadrequest.cpp
  */

#include <fstream>
#include <sstream>
#include "turnuploadrequest.h"
#include "nls.h"
#include "filenames.h"
#include "net/mimebuilder.h"
#include "net/smtp.h"
#include "net/http.h"
#include "net/mailtemplate.h"
#include "game.h"
#include "race.h"
#include "util.h"
#include "version.h"

TurnUploadRequest::TurnUploadRequest(Game& game, Race& race)
    : m_Game(game), m_Race(race)
{ }

TurnUploadRequest::~TurnUploadRequest()
{ }

void
TurnUploadRequest::run()
{
    // set to unknown upload result
    m_Parser.setUnknown();

    // figure out URL
    {
        Url u;
        if (u.parse(m_Race.getUploadUrl())) {
            if (u.getProtocol() == "file") {
                doCopy(getFileNameFromUrl(u.getHost(), u.getPath()));
                return;
            } else if (u.getProtocol() == "http" || u.getProtocol() == "http-post") {
                doUpload(u);
                return;
            } else {
                setResultMessage(_("Only HTTP upload currently supported"));
                setStatus(StatusFailed);
                return;
            }
        }
    }

    // or do we want to mail?
    {
        MailTemplate t;
        if (t.parse(m_Race.getUploadUrl())) {
            doMail(t);
            return;
        }
    }

    setResultMessage(_("Cannot handle upload address '") + m_Race.getUploadUrl() + "'");
    setStatus(StatusFailed);
}

std::string
TurnUploadRequest::getName()
{
    return _("Upload turn for ") + m_Game.getName();
}

void
TurnUploadRequest::doCopy(std::string dir)
{
    // upload race's turn file
    std::string basename = "player" + itoa(m_Race.getRaceNumber()) + ".trn";
    std::string tname = makeFileName(m_Game.getLocalDirectory(), basename);
    std::ifstream trn(tname.c_str(), std::ifstream::binary);
    if (!trn) {
        setResultMessage("'" + tname + _("' does not exist, run maketurn first"));
        setStatus(StatusFailed);
        return;
    }

    // output file
    std::string destname = makeFileName(dir, basename);
    std::ofstream out(destname.c_str(), std::ofstream::binary);
    if (!out) {
        setResultMessage("'" + destname + _("' cannot be created"));
        setStatus(StatusFailed);
        return;
    }

    // copy
    out << trn.rdbuf();
    if (!out) {
        setResultMessage(_("Error copying '") + basename + "'");
        setStatus(StatusFailed);
        return;
    }

    setResultMessage(_("Turn successfully copied"));
    m_Parser.setCopied();
}

void
TurnUploadRequest::doUpload(Url& u)
{
    // upload race's turn file
    std::string basename = "player" + itoa(m_Race.getRaceNumber()) + ".trn";
    std::string tname = makeFileName(m_Game.getLocalDirectory(), basename);
    std::ifstream trn(tname.c_str(), std::ifstream::binary);
    if (!trn) {
        setResultMessage("'" + tname + _("' does not exist, run maketurn first"));
        setStatus(StatusFailed);
        return;
    }

    // build packet to upload
    MimeBuilder builder("multipart/form-data");
    MimeParameterEncoder mpe(builder, MimeParameterEncoder::EncodeNone);
    u.trimParameters(mpe);

    // Now add the file
    builder.addBoundary();
    builder.addHeader("Content-Disposition",
                      std::string("form-data; name=\"") + m_Race.getUploadId() + "\"; filename=\"" + basename + "\"");
    builder.addHeader("Content-Type", "application/octet-stream");
    char buf[4096];
    int count;
    while ((count = trn.rdbuf()->sgetn(buf, sizeof(buf))) > 0) {
        builder.addRawData(buf, count);
    }
    builder.addBoundary();
    builder.finish();

    // Upload
    Http req;
    std::stringstream output;
    if (req.post(u, builder, output)) {
        if (req.isSuccessfulReply()) {
            setResultMessage(_("Turn successfully uploaded"));
            m_Parser.parse(output.str());
        } else {
            setResultMessage(_("Server sent error: ") + req.getReplyLine());
            setStatus(StatusFailed);
        }
    } else {
        setResultMessage(_("Network error: ") + req.getErrorMessage()
                         + ", " + req.getServerErrorMessage());
        setStatus(StatusFailed);
    }
}

void
TurnUploadRequest::doMail(MailTemplate& t)
{
    std::string from_addr = Smtp::getFromAddress();

    // mail race's turn file
    std::string basename = "player" + itoa(m_Race.getRaceNumber()) + ".trn";
    std::string tname = makeFileName(m_Game.getLocalDirectory(), basename);
    std::ifstream trn(tname.c_str(), std::ifstream::binary);
    if (!trn) {
        setResultMessage("'" + tname + _("' does not exist, run maketurn first"));
        setStatus(StatusFailed);
        return;
    }

    // build packet to upload
    MimeBuilder mb("multipart/mixed");
    mb.addHeader("From", from_addr);
    mb.addHeader("To", t.getAddress());
    if (t.getSubject().size())
        mb.addHeader("Subject", t.getSubject());
    else
        mb.addHeader("Subject", "Turn file for " + m_Game.getName());
    mb.addHeader("X-Mailer", USER_AGENT);
    mb.addHeader("MIME-Version", "1.0");

    /* Text part */
    mb.addBoundary();
    mb.addHeader("Content-Type", "text/plain; charset=iso-8859-1");
    mb.addHeader("Content-Disposition", "inline");
    mb.addHeader("Content-Transfer-Encoding", "quoted-printable");

    if (t.getBody().size()) {
        mb.addLineQP(t.getBody());
    } else {
        mb.addLineQP("Please find attached the turn file for '" + m_Game.getName() + "' player " +
                     itoa(m_Race.getRaceNumber()) + ".");
    }
    mb.addLineQP("");

    /* File part */
    mb.addBoundary();
    mb.addHeader("Content-Type", "application/octet-stream");
    mb.addHeader("Content-Disposition", "attachment; filename=\"" + basename + "\"");
    mb.addHeader("Content-Transfer-Encoding", "base64");
    char buf[4096];
    int count;
    while ((count = trn.rdbuf()->sgetn(buf, sizeof(buf))) > 0) {
        mb.addBase64(buf, count);
    }
    mb.addBoundary();
    mb.finish();

    Smtp mailer;
    if (mailer.sendMail(from_addr, t.getAddress(), mb)) {
        setResultMessage(_("Turn mailed successfully"));
        m_Parser.setMailed();
    } else {
        setResultMessage(_("Network error: ") + mailer.getErrorMessage()
                         + ", " + mailer.getServerErrorMessage());
        setStatus(StatusFailed);
    }
}
