/**
  *  \file execute.cpp
  *  \brief Executing stuff
  */
#include <string>
#include <fstream>
#include <sstream>
#include "execute.h"
#include "game.h"
#include "selection.h"
#include "race.h"
#include "action.h"
#include "net/smtp.h"
#include "net/http.h"
#include "net/mailtemplate.h"
#include "nls.h"
#include "net/url.h"
#include "filenames.h"
#include "workerthread.h"
#include "filedownloadrequest.h"
#include "turnuploadrequest.h"
#include "expander.h"
#include "multirequest.h"
#include "basicrequests.h"
#include "program.h"

WorkerThread network_thread;


/* Main Entry Point */

Executer::Executer()
    : m_NumJobs(0)
{ }

void
Executer::runAction(HWND window, Selection& sel)
{
    if (!sel.getSelectedAction())
        return;

    Action& a = *sel.getSelectedAction();

    // Build request container
    std::string name = a.getDescription();
    if (sel.getSelectedGame()) {
        name += _(" for ");
        name += sel.getSelectedGame()->getName();
    }
    MultiRequest mr(name);

    // Figure out commands to perform
    StringList stuff;
    Expander e;
    sel.setVariables(e);
    e.expandToList(stuff, a.getCommandLine());

    if (stuff.empty()) {
        MessageBox(window, _("This command is currently not possible."), name.c_str(), MB_ICONSTOP | MB_OK);
        return;
    }

    // Do it
    for (StringList::iterator i = stuff.begin(); i != stuff.end(); ++i) {
        std::string s = *i;
        std::string verb = eatWord(s);

        // now verb=command, s=arg
        if (verb == "shell") {
            while (s.size() && s[0] == '-')
                eatWord(s);
            // MessageBox(window, (_("Would now run '") + s + "'\r\n").c_str(), a.getDescription().c_str(),
            //            MB_ICONSTOP | MB_OK);
            std::string title = a.getName();
            std::string cwd = a.getProgram().getDirectory();
            STARTUPINFO si;
            PROCESS_INFORMATION pi;
            std::memset(&si, 0, sizeof(si));
            si.cb      = sizeof(si);
            si.lpTitle = (char*) title.c_str();
            si.dwFlags = 0;
            if (!CreateProcess(0, (char*) s.c_str(), 0, 0, FALSE, CREATE_NEW_CONSOLE, 0,
                               cwd.size() ? cwd.c_str() : 0, &si, &pi)) {
                MessageBox(window, (_("Error ") + itoa(GetLastError())
                                    + _(" running program ") + s).c_str(),
                           a.getDescription().c_str(),
                           MB_ICONSTOP | MB_OK);
            }
        } else if (verb == "browser") {
            if (s.size()) {
                ShellExecute(window, "open", s.c_str(), 0,
                             sel.getSelectedGame() ? sel.getSelectedGame()->getLocalDirectory().c_str() : ".",
                             0);
            } else {
                MessageBox(window, _("This action would open a web site, but the URL was not specified."),
                           _("Configuration error"),
                           MB_ICONSTOP | MB_OK);
            }
        } else if (verb == "builtin-download-rst") {
            if (sel.getSelectedGame()) {
                doDownloadResults(*sel.getSelectedGame(), mr);
            }
        } else if (verb == "builtin-download-spec") {
            if (sel.getSelectedGame()) {
                if (s.size()) {
                    doDownloadFile(s, *sel.getSelectedGame(), mr);
                } else {
                    doDownloadSpec(*sel.getSelectedGame(), mr);
                }
            }
        } else if (verb == "builtin-upload-trn") {
            if (sel.getSelectedGame()) {
                doUploadTurns(*sel.getSelectedGame(), mr);
            }
        } else {
            MessageBox(window, (_("The command '") + verb + _("' specified by this action is not supported.")).c_str(),
                       _("Configuration error"), MB_ICONSTOP | MB_OK);
        }
    }

    if (mr.getNumRequests()) {
        MultiRequest* the_req = new MultiRequest(mr);
        the_req->addReadOperation(sel.getSelectedGame());
        the_req->addReadOperation(&sel.getSelectedAction()->getProgram());
        the_req->acquireLocks();
        pushRequest(sel, the_req);
    }
}

/** Check whether the specified action is still executing.
    \param sel Action to check */
bool
Executer::isPendingAction(const Selection& sel) const
{
    for (std::list<Selection>::const_iterator i = m_PendingRequests.begin(); i != m_PendingRequests.end(); ++i)
        if (*i == sel)
            return true;
    return false;
}

void
Executer::pushRequest(const Selection& sel, Request* r)
{
    network_thread.pushRequest(r);
    m_PendingRequests.push_back(sel);
    ++m_NumJobs;
}

Request*
Executer::popFinishedRequest()
{
    Request* r = network_thread.popFinishedRequest();
    if (r) {
        --m_NumJobs;
        m_PendingRequests.pop_front();
    }
    return r;
}
